PROGRAM sistema
IMPLICIT NONE

INTEGER, PARAMETER:: n=500, fdat=11, fsal=12
DOUBLE PRECISION a(n,n), aa(n,n), b(n), bb(n), x(n), xsol(n)
INTEGER ipiv(n)
DOUBLE PRECISION err, t1, t2
INTEGER info
INTEGER i, tipo
CHARACTER(LEN=30) pc(2)
pc(1) = 'I7-1065G7, 1.30GHz, 16Gb RAM'
pc(2) = 'Quad Q2800, 2.33GHz, 4Gb RAM'

OPEN (fdat, FILE='sistema-NOFORM.dat',FORM='UNFORMATTED') 
OPEN (fsal, FILE='result2.sal', POSITION='append')
WRITE (*,'(A,2(/5X,I1,'': '',A)/10X,A)',ADVANCE='NO') 'Tipo de PC = ', &
      (i,pc(i),i=1,2), 'Opcion = '
READ*, tipo      
WRITE (fsal,'(/2A/)') 'Tipo de PC: ', pc(tipo)      

WRITE (fsal,'(A,I0,3X,A)') 'n = ', n, 'Lee A, b de fichero NO FORMATEADO'

! Generacion del sistema

CALL RANDOM_NUMBER (a)
a = 0.1d0 + 0.8d0*a
CALL RANDOM_NUMBER (xsol)
xsol = 0.1d0 + 0.8d0*xsol   ! verdadera solucion
b = MATMUL(a,xsol)
aa = a ; bb = b
DO i = 1, n
  WRITE (fdat) b(i), a(i,1:n)
ENDDO
REWIND(fdat)

! NR

CALL cpu_time (t1)
DO i = 1, n
  READ (fdat) b(i), a(i,1:n)
ENDDO
CALL lusol (n, a, n, b, info)   ! en b devuelve la solucion
err = SUM(ABS(b-xsol))
CALL cpu_time (t2)
WRITE (fsal,'(A,E20.10,A,F8.3)') 'error     NR = ', err, '   tiempo     NR = ', t2-t1
PRINT '(A,E20.10,A,F8.3)', 'error     NR = ', err, '   tiempo     NR = ', t2-t1

! LAPACK

REWIND(fdat)
DO i = 1, n
  READ (fdat) b(i), a(i,1:n)
ENDDO
CALL cpu_time (t1)
CALL dgetrf (n, n, a, n, ipiv, info)   ! a = LU, diag(L) = 1
CALL dgetrs ('N', n, 1, a, n, ipiv, b, n, info)
err = SUM(ABS(b-xsol))
CALL cpu_time (t2)
WRITE (fsal,'(A,E20.10,A,F8.3)') 'error LAPACK = ', err, '   tiempo LAPACK = ', t2-t1
PRINT '(A,E20.10,A,F8.3)', 'error LAPACK = ', err, '   tiempo LAPACK = ', t2-t1

ENDPROGRAM sistema

! ======================================================================
! RESOLUCION DEL SISTEMA Ax = b. Adaptacion de Numerical Recipes ludcmp
! ----------------------------------------------------------------------

! INTEGER:: ia <=> dimension de filas de a
! DOUBLE PRECISION:: a(ia,*) <=> matriz del sistema
! INTEGER:: n <=> numero de ecuaciones
! DOUBLE PRECISION:: b(*) <=> vector del lado derecho, en salida b es 
!                              la solucion
! INTEGER:: ierlu <=> indicador de error
!           ierlu = 0 : solucion correcta
!           ierlu = -1: pivote nulo; no hay solucion o no es unica
! ----------------------------------------------------------------------

SUBROUTINE lusol (ia, a, n, b, ierlu)
IMPLICIT NONE

! Argumentos

INTEGER, INTENT(IN):: ia, n
INTEGER, INTENT(OUT):: ierlu
DOUBLE PRECISION, INTENT(INOUT):: a(ia,*), b(*)

! Variables locales

INTEGER i, imax, j, k, ii, ll
INTEGER ind(n)
DOUBLE PRECISION vv(n)
DOUBLE PRECISION aamax, dum, suma
DOUBLE PRECISION, PARAMETER:: eps=1.d-20

! ----------------------------
! Primera sentencia ejecutable

ierlu = 0
DO i = 1, n
  aamax = MAXVAL(ABS(a(i,1:n)))
  IF (aamax < eps) THEN
    ierlu = -1 ; RETURN
  ENDIF
  vv(i) = 1.d0 / aamax
ENDDO

DO j = 1, n

  DO i = 1, j-1
    suma = a(i,j)
    DO k = 1, i-1 ; suma = suma - a(i,k)*a(k,j) ; ENDDO
    a(i,j) = suma
  ENDDO

  aamax = 0.d0
  DO i = j, n
    suma = a(i,j)
    DO k = 1, j-1 ; suma = suma - a(i,k)*a(k,j) ; ENDDO
    a(i,j) = suma
    dum = vv(i) * ABS(suma)
    IF (dum >= aamax) THEN
      imax = i ; aamax = dum
    ENDIF
  ENDDO

  IF (j /= imax) THEN
    DO k = 1, n
      dum = a(imax,k) ; a(imax,k) = a(j,k) ; a(j,k) = dum
    ENDDO
    vv(imax) = vv(j)
  ENDIF

  ind(j) = imax
  IF (ABS(a(j,j)) < eps) a(j,j) = eps
  IF (j /= n) a(j+1:n,j) = a(j+1:n,j) / a(j,j)

ENDDO

ii = 0
DO i = 1, n
  ll = ind(i) ; suma = b(ll) ; b(ll) = b(i)
  IF (ii /= 0) THEN
    DO j = ii, i-1 ; suma = suma - a(i,j)*b(j) ; ENDDO
  ELSEIF (ABS(suma) >= eps) THEN
    ii = i
  ENDIF
  b(i) = suma
ENDDO

DO i = n, 1, -1
  suma = b(i)
  DO j = i+1, n ; suma = suma - a(i,j)*b(j) ; ENDDO
  b(i) = suma / a(i,i)
ENDDO

ENDSUBROUTINE lusol
